function varargout=mva_component_examples(what,varargin)
% function varargout=mva_component_examples(what,varargin)
% This contains the simulations presented in
% Diedrichsen, Ridgeway, Friston & Wiestler (2011). 
%
% For the full execution, you need util & graph toolbox from: 
% http://www.icn.ucl.ac.uk/motorcontrol/toolboxes/matlab_toolboxes.htm
% 
% To Replicate the simulations presented in the article 
% Figure 2: 
%   T=mva_component_examples('Similarity_noise');
%   mva_component_examples('plot_Figure2',T);
% Figure 3: 
%   T=mva_component_examples('Similarity_commonfactor');
%   mva_component_examples('plot_Figure3',T);
% Figure 4: 
%   T=mva_component_examples('TwoFactor');
%   mva_component_examples('plot_Figure4',T);
% Figure 5: 
%   T=mva_component_examples('Voxelselection');
%   mva_component_examples('plot_Figure5',T);
% Figure 8:
%   T=mva_component_examples('SpatialKernel',T);
%   mva_component_examples('plot_Figure8',T);
% 
% (c) 2011 Joern Diedrichsen, j.diedrichsen@ucl.ac.uk
% 
switch (what)
    case '3d_sphere'               % Make a distance matrix based on a 3D-sphere
        r=varargin{1};
        [X,Y,Z]=ndgrid([-r:r],[-r:r],[-r:r]) ;
        D=[X(:) Y(:) Z(:)];
        d=sqrt(sum(D.*D,2));
        D=D(d<=r,:);
        Xd=bsxfun(@minus,D(:,1),D(:,1)');
        Yd=bsxfun(@minus,D(:,2),D(:,2)');
        Zd=bsxfun(@minus,D(:,3),D(:,3)');
        C=sqrt(Xd.^2+Yd.^2+Zd.^2);
        varargout={C};      
    case 'data1'                   % Data for one-factorial design
        D=varargin{1};
        C=varargin{2};
        Z=varargin{3};
        
        % Z-matrix
        trueU=randn(1,size(C.cholST,1))*C.cholST;
        trueU=reshape(trueU,D.P,size(Z,2))';
        
        err=randn(D.K.*D.L,D.P)*C.cholSigE*sqrt(D.var_e);
        
        y=Z*trueU+err;
        c=[];
        for i=1:D.K
            c=[c ones(1,D.L)*i];
        end;
        varargout={y,Z,c,trueU};
    case 'cov2_sim'                % Covariance for Kx2 design with matching similarity
          % Generate a 2xK two-factorial design
        % ------------------------------------------------------
        % D.var_a1 = variance of the condition effect (1)
        % D.var_a2 = variance of the condition effect (2)
        % D.corr_a =
        % D.var_b1 = variance of b in condition 1
        % D.var_b2 = variance of b in condition 2
        % D.corr_b = correlation of matching b patterns
        % D.var_e = noise variance
        % D.L = Number of trials per condition
        % D.K = number of conditions
        % D.P = Number of voxels      
        D=varargin{1};
        C=varargin{2};
        D.P=size(C.dist,1);
        
        % Make True G
        D.cov_a=D.corr_a*sqrt(D.var_a1)*sqrt(D.var_a2);
        D.cov_b=D.corr_b*sqrt(D.var_b1)*sqrt(D.var_b2);
        C.trueGA=[D.var_a1 D.cov_a; D.cov_a D.var_a2];
        nc=D.K;
        C.trueGB=blockdiag(eye(D.K)*D.var_b1,eye(D.K)*D.var_b2)+...
            (diag(ones(nc,1),nc)+diag(ones(nc,1),-nc))*D.cov_b;
        C.trueG=blockdiag(C.trueGA,C.trueGB);
        C.cholGA=cholcov(C.trueGA);
        C.cholGB=cholcov(C.trueGB);
        
        % make spatial cholinsky matrices
        C.SigA=exp(-C.dist.^2/(2*D.s_a.^2));
        C.SigB=exp(-C.dist.^2/(2*D.s_b.^2));
        C.SigE=exp(-C.dist.^2/(2*D.s_e.^2));
        C.cholSigA=cholcov(C.SigA);
        C.cholSigB=cholcov(C.SigB);
        C.cholSigE=cholcov(C.SigE);
        
        % Make true G
        Z=[kron(eye(2),ones(D.K*D.L,1)) kron(eye(2*D.K),ones(D.L,1))];
        
        C.cholST=blockdiag(kron(C.cholGA,C.cholSigA),kron(C.cholGB,C.cholSigB));
        varargout={D,C,Z};
    case 'data2_sim'               % Data for Kx2 design
        D=varargin{1};
        C=varargin{2};
        Z=varargin{3};
        
        % Z-matrix
        trueU=randn(1,size(C.cholST,1))*C.cholST;
        trueU=reshape(trueU,D.P,size(Z,2))';
        
        err=randn(2*D.K.*D.L,D.P)*C.cholSigE*sqrt(D.var_e);
        
        y=Z*trueU+err;
        c=[];
        for i=1:D.K
            c=[c ones(1,D.L)*i];
        end;
        varargout={y,Z,c,trueU};    
    case 'Similarity_noise'        % Example is for Figure 2 (Similarity structure and noise) 
        C.dist=mva_component_examples('3d_sphere',3.5);       % 3.5 voxel Sphere 
        D.P=size(C.dist,1);                     % Number of voxels
        D.K=3;                                  % 3 conditions 
        D.L=5;                                  % 4 repetions per trial 
        D.var_e=10;
        D.s_a=0.0001;                           % Spatial kernel 
        D.s_e=0.0001;
        X=2*[0 0;                               % true distance structure 
            0.5 0;
            0.6 0;
            ];
        C.trueG=1-squareform(pdist(X));
        C.SigA=exp(-C.dist.^2/(2*D.s_a.^2));
        C.SigE=exp(-C.dist.^2/(2*D.s_e.^2));
        C.cholSigA=cholcov(C.SigA);
        C.cholSigE=cholcov(C.SigE);
        C.cholG=cholcov(C.trueG);
        C.cholST=kron(C.cholG,C.cholSigA);
        Z=kron(eye(D.K),ones(D.L,1));
        
        T=[];
        
        for e=[0.5 1 4 10]          % Loop over different noise structures 
            for n=1:100
                D.var_e=e;
                [y,Z,c,u]=mva_component_examples('data1',D,C,Z); % generate the data 
                
                % These are the correlations of the true pattern components
                actG=corrcoef(u');
                
                % Normal sample correlations on means
                b=inv(Z'*Z)*Z'*y;
                SC=corrcoef(b');

                % Decomposition of pattern
                Ac={};
                for i=1:D.K
                    for j=i:D.K
                        Ac{end+1}=zeros(D.K,D.K);
                        Ac{end}(i,j)=1;
                        Ac{end}(j,i)=1;
                    end;
                end;                
                tic;[G,h,u,l,D.niter]=mvpattern_covcomp(y,Z,'Ac',Ac,'num_iter',2000);D.time=toc;
                D.G_mv=G(:)';
                D.G_true=C.trueG(:)';
                D.pdist_theo=squareform(1-corrcov(C.trueG));  % These are the theoretical correlations 
                D.pdist_true=squareform(1-actG);     % True correlations, measured on the (unknown) pattern components
                D.pdist_mv=squareform(1-corrcov(G)); % Corrected estimates from the component model 
                D.pdist_corr=squareform(1-SC);       % Sample correlations 
                T=addstruct(T,D);
            end;
        end;
        varargout={T,Z,y,u};
    case 'plot_Figure2'            % Plots Figure 2
        T=varargin{1};
        true=[0 -0.2 0.8];
        for i=1:3
            subplot(2,3,i);
            lineplot(T.var_e,[1-T.pdist_corr(:,i) 1-T.pdist_mv(:,i) ],'style_thickline','leg',{'Sample corr','component'});
            drawline(true(i),'dir','horz');
            set(gca,'YLim',[-0.3 0.95]);
        end;
                
        Dtrue=squareform(mean(T.pdist_true)); % These are the actual correlations (true) 
        Dcorr=squareform(mean(T.pdist_corr)); % Sample correlations 
        Dmv=squareform(mean(T.pdist_mv));     % Component model estimates 
        
        nam={'1','2','3'};
        subplot(2,3,4);
        [Y_true,eigvals] = cmdscale(Dtrue);
        Y_true=[Y_true zeros(size(Y_true,1),1)];
        plot(Y_true(:,1),Y_true(:,2),'ko');
        axis([-0.9 0.9 -0.67 0.67]);
                
        subplot(2,3,5);
        [Y_corr,eigvals] = cmdscale(Dcorr);
        plot(Y_corr(:,1),Y_corr(:,2),'ko'); 
        axis([-0.9 0.9 -0.67 0.67]);
        
        subplot(2,3,6);
        [Y_mv,eigvals] = cmdscale(Dmv);
        plot(Y_mv(:,1),Y_mv(:,2),'ko'); 
        axis([-0.9 0.9 -0.67 0.67]);       
    case 'Similarity_commonfactor' % Example is for Figure 3 (Similarity structure and common factor)
        % Make scaling structure
        C.dist=mva_component_examples('3d_sphere',3.5);
        D.P=size(C.dist,1);
        D.K=4;              % Number of conditions 
        D.L=5;              % Repetition per conditions 
        D.var_e=3;          % Noise variance 
        D.var_a=4;           % Variance of common factor 
        D.s_a=0.0001;       % No spatial autocorrelation 
        D.s_e=0.0001;
        X=2*[0 0;
            0.5 0;
            0.6 0;
            ];
        Gtheo=1-squareform(pdist(X));       % Variance of single factor is set to 1
        C.trueG=blockdiag(D.var_a,Gtheo);   % True variance structure G
   
        C.SigA=exp(-C.dist.^2/(2*D.s_a.^2));       % Spatial kernel: uncorrelated data
        C.SigE=exp(-C.dist.^2/(2*D.s_e.^2));
        C.cholSigA=cholcov(C.SigA);     
        C.cholSigE=cholcov(C.SigE);
        C.cholG=cholcov(C.trueG);
        C.cholST=kron(C.cholG,C.cholSigA);
        Za=kron(eye(D.K),ones(D.L,1));
        Z=[ones(D.K.*D.L,1) Za(:,2:4)];
        T=[];
        
        for e=[0.5 1 4 10]
            for n=1:200
                D.var_e=e;
                
                [y,Z,c,u]=mva_component_examples('data1',D,C,Z); % Generate data 

                % Actual sample correlations
                actG=corrcoef(u(2:4,:)');
                
                % Normal sample correlations on means
                b=inv(Za'*Za)*Za'*y;
                SC=corrcoef(b(2:4,:)');
                
                % With subtracting the control condition
                bsub=b(2:4,:)'-repmat(b(1,:)',1,3);
                SCsub=corrcoef(bsub);
                
                % Do the component model estimation 
                Ac={};
                Ac{1}=zeros(4,4);
                Ac{1}(1,1)=1;
                for i=1:3
                    for j=i:3
                        Ac{end+1}=zeros(4,4);
                        Ac{end}(i+1,j+1)=1;
                        Ac{end}(j+1,i+1)=1;
                    end;
                end;                
                [G,h,u,l,niter]=mvpattern_covcomp(y,Z,'Ac',Ac,'num_iter',2000);

                D.Ga_mv=G(1,1);
                D.Ga_true=C.trueG(1,1);
                a=G(2:4,2:4);D.G_mv=a(:)';
                a=C.trueG(2:4,2:4);D.G_true=a(:)';
    
                D.pdist_theo=squareform(1-corrcov(C.trueG));  % These are the theoretical correlations 
                D.pdist_true=squareform(1-actG);     % True correlations, measured on the (unknown) pattern components
                D.pdist_corr=squareform(1-SC);       % Sample correlations 
                D.pdist_sub=squareform(1-SCsub);     % Mean subtracted distances 
                D.pdist_mv=squareform(1-corrcov(G(2:4,2:4))); % Corrected distances from component model 
                T=addstruct(T,D);
            end;
        end;
        varargout={T,Z,y,u};
    case 'plot_Figure3'            % Plots Figure 3
        T=varargin{1};
        true=[0 -0.2 0.8];
        for i=1:3
            subplot(2,3,i);
            lineplot(T.var_e,[1-T.pdist_corr(:,i) 1-T.pdist_mv(:,i) 1-T.pdist_sub(:,i)],'style_thickline','leg',{'Sample corr','component','subtracted corr'});
            drawline(true(i),'dir','horz');
            set(gca,'YLim',[-0.3 0.95]);
        end;
                
        Dtrue=squareform(mean(T.pdist_true)); % These are the actual correlations (true) 
        Dcorr=squareform(mean(T.pdist_corr)); % Sample correlations 
        Dmv=squareform(mean(T.pdist_mv));     % Component model estimates 
        Dsub=squareform(mean(T.pdist_sub));
        
        nam={'1','2','3'};
        subplot(2,3,4);
        [Y_corr,eigvals] = cmdscale(Dcorr);
        plot(Y_corr(:,1),Y_corr(:,2),'ko');
        axis([-0.9 0.9 -0.67 0.67]);
        xlabel('sample Correlations'); 
        
        subplot(2,3,5);
        [Y_sub,eigvals] = cmdscale(Dsub);
        Y_sub=[Y_sub zeros(size(Y_sub,1),1)];
        plot(Y_sub(:,1),Y_sub(:,2),'ko');
        axis([-0.9 0.9 -0.67 0.67]);
        xlabel('Subtracted Correlations'); 

        subplot(2,3,6);
        [Y_mv,eigvals] = cmdscale(Dmv);
        plot(Y_mv(:,1),Y_mv(:,2),'ko'); 
        axis([-0.9 0.9 -0.67 0.67]);      
        xlabel('Component model'); 
    case 'TwoFactor'               % Example from 2-factor design 
        % Settings
        D.K = 4;        % Number of categories 
        D.L = 7;        % Number of repetitions 
        D.var_e = 0.5;  % Error variances
        D.var_a1 = 2;   % Variance of the common factor condition 1 
        D.var_a2 = 2;   % 
        D.var_b1 = 1;   % variance of factor B, condition 1 
        D.var_b2 = 1;
        D.corr_b = 0.5;   % true correlation between components of the same category across condition 
        D.corr_a = 0;   % True correlation between the two common factors 
        T=[];
        
        D.s_a=0.001;
        D.s_b=0.001;
        D.s_e=0.001;
        
        D.N=D.L*D.K*2;
        C.dist=mva_component_examples('3d_sphere',3);
        D.P=size(C.dist,1);
        
        var_e=[0.5  1 4 8];
        var_a=2; % [0.5 1 2 3];
        corr_a=[0 0.3 0.6 0.9];
        
        % Testing decompositon 
        for evar=var_e
            for ac=corr_a
                D.var_e=evar;
                D.corr_a=ac;
                
                [D,C,Z]=mva_component_examples('cov2_sim',D,C);
                
                for n=1:40
                    [y,Z,c]=mva_component_examples('data2_sim',D,C,Z);
                                        
                    % Normal correlational analysis
                    A=pinv(Z(:,3:end))*y;
                    CorM=corrcoef(A');
                    D.C1_corrU_b=mean(diag(CorM,4)); % Correlations for same level on B 
                    D.C1_corrU_nonb=mean([diag(CorM,5);diag(CorM,6);diag(CorM,7)]); % Correlations for non-matching on factor B 
                    
                    % Corrected for factor A 
                    A=pinv(Z)*y;
                    CorM=corrcoef(A');
                    [D.C2_corrU_a,D.C2_corrU_b]=corrM2coeff(D,CorM); 
                    
                    % mv_pattern with covariance set to zero
                    Kb=D.K*2;
                    a=0.9;
                    Ac={};
                    Ac{1}=blockdiag(eye(2),zeros(Kb,Kb));                                        % Var_a
                    Ac{2}=Ac{1}+a*blockdiag((1-eye(2)),zeros(Kb,Kb));                            % Cov_a
                    Ac{3}=blockdiag(zeros(2,2),eye(D.K),zeros(D.K,D.K));                         % var_b1
                    Ac{4}=blockdiag(zeros(2,2),zeros(D.K,D.K),eye(D.K));                         % var_b2
                    Ac{5}=blockdiag(zeros(2,2),eye(Kb))+a*blockdiag(zeros(2,2),diag(ones(D.K,1),D.K)+diag(ones(D.K,1),-D.K)); % Cov B   % Cov_b
                    tic;
                    [G,h,u,l,D.niter]=mvpattern_covcomp(y,Z,'num_iter',1000,'Ac',Ac,'TolL',1e-6,'accel_method','Aitken');
                    D.time=toc; 
                    D.D2_var_a=G(1,1);
                    D.D2_cov_a=G(1,2);
                    D.D2_var_b1=G(3,3);
                    D.D2_var_b2=G(D.K+3,D.K+3);
                    D.D2_cov_a=G(1,2);
                    D.D2_corr_a=D.D2_cov_a./D.D2_var_a;
                    D.D2_cov_b=G(3,D.K+3);
                    D.D2_corr_b=D.D2_cov_b/sqrt(D.D2_var_b1*D.D2_var_b2);
                    T=addstruct(T,D);
                end;
            end;
        end;
        varargout={T};   
    case 'plot_Figure4'            % Plots Figure 4
        T=varargin{1};
        figure(1);
        M=4;
        lim=[0 0.8];
        subplot(1,M,1);
        lineplot(T.var_e,T.C1_corrU_b,'style_thickline','split',T.corr_a,'leg','auto');
        drawline(T.corr_b(1),'dir','horz');
        set(gca,'YLim',lim);
        title('Normal correlation coeffiecient');
        subplot(1,M,2);
        lineplot(T.var_e,T.C1_corrU_b-T.C1_corrU_nonb,'style_thickline','split',T.corr_a);
        drawline(T.corr_b(1),'dir','horz');
        set(gca,'YLim',lim);
        title('Difference');
        subplot(1,M,3);
        lineplot(T.var_e,T.C2_corrU_b,'style_thickline','split',T.corr_a);
        drawline(T.corr_b(1),'dir','horz');
        set(gca,'YLim',lim );
        title('Mean subtracted');
        subplot(1,M,4);
        lineplot(T.var_e,T.D2_corr_b,'style_thickline','split',T.corr_a);
        drawline(T.corr_b(1),'dir','horz');
        set(gca,'YLim',lim);
        title('Corrected');
        set(gcf,'PaperPosition',[2 2 8 3]);
        wysiwyg;
        % ----------------------------------------------------
        % Encoding the correlation of two factors either as covariance, or
        % as a common factor (positive or negative, allowing us to use
        % spm_reml_sc
    case 'Voxelselection'          % Mixture of informative and non-informative voxels
        % Settings
        D.K = 4;
        D.L = 7;
        D.var_e = 4;
        D.var_a1 = 2;
        D.var_a2 = 2;
        D.var_b1 = 1;
        D.var_b2 = 1;
        D.corr_b = 0.5;
        D.corr_a = 0;
        T=[];
        
        D.s_a=0.0001;       % No spatial autocorrelation 
        D.s_e=0.0001;
        D.s_b=0.0001;       
        
        D.N=D.L*D.K*2;
        C.dist=mva_component_examples('3d_sphere',3.5);
        D.P=size(C.dist,1);
        
        bcorr=0.5;
        var_e=4;
        prop=[0 0.25 0.5 0.75]; % Proportion of informative voxels 

   
        [D,C,Z]=mva_component_examples('cov2_sim',D,C);

        
        for propI=prop
            D.prop=propI;
            numUnInform=round(D.P*D.prop);
            
            for n=1:60
                [y,Z,c]=mva_component_examples('data2_sim',D,C,Z);
                err=randn(D.N,D.P)*C.cholSigE*sqrt(D.var_e);
                y(:,1:numUnInform)=err(:,1:numUnInform); % Kill information from non-informative voxels 
            
                % Pattern decomposition 
                Kb=D.K*2;
                a=0.9;
                Ac={};
                Ac{1}=blockdiag(1,0,zeros(Kb,Kb));                                        % Var_a1
                Ac{2}=blockdiag(0,1,zeros(Kb,Kb));                                         % Var_a2
                CovCom=chol([1 a;a 1]);
                Ac{3}=blockdiag(CovCom,zeros(Kb,Kb));                            % Cov_a
                Ac{4}=blockdiag(zeros(2,2),eye(D.K),zeros(D.K,D.K));                         % var_b1
                Ac{5}=blockdiag(zeros(2,2),zeros(D.K,D.K),eye(D.K));                         % var_b2
                Ac{6}=blockdiag(zeros(2,2),eye(Kb))+a*blockdiag(zeros(2,2),diag(ones(D.K,1),D.K)+diag(ones(D.K,1),-D.K));  % Cov_b
                [G,h,u,l,n]=mvpattern_covcomp(y,Z,'num_iter',1000,'Ac',Ac,'TolL',1e-6,'accel_method','Aitken');
                D.D1_F=l(end);
                D.D1_var_e=h(end,end);
                D.D1_var_a1=G(1,1);
                D.D1_var_a2=G(2,2);
                D.D1_cov_a=G(1,2);
                D.D1_corr_a=D.D1_cov_a/sqrt(D.D1_var_a1*D.D1_var_a2);
                
                D.D1_var_b1=G(3,3);
                D.D1_var_b2=G(D.K+3,D.K+3);
                D.D1_cov_b=G(3,D.K+3);
                D.D1_corr_b=D.D1_cov_b/sqrt(D.D1_var_b1*D.D1_var_b2);
                
                % Simple correlation
                A=pinv(Z(:,3:end))*y;
                CorM=corrcoef(A');
                D.C1_corrU_b=mean(diag(CorM,4));
                D.C1_corrU_nonb=mean([diag(CorM,5);diag(CorM,6);diag(CorM,7)]);
                
                % Corrected for main effect
                A=pinv(Z)*y;
                CorM=corrcoef(A');
                [D.C2_corrU_a,D.C2_corrU_b]=corrM2coeff(D,CorM);
                
                T=addstruct(T,D);
            end;
        end;
        varargout={T};
    case 'plot_Figure5'            % Plots Figure 5
        T=varargin{1};
        lineplot(T.prop,[(T.C1_corrU_b-T.C1_corrU_nonb) T.C2_corrU_b  T.D1_corr_b],'style_thickline','leg',{'difference same-different','Mean Subtracted','Component model'});
        set(gca,'YLim',[0 0.6]);
        set(gcf,'PaperPosition',[2 2 6 5]);
        wysiwyg;
        drawline(0.5,'dir','horz');     
    case 'SpatialKernel'           % Simulations with spatial autocorrelation
        % Settings
        D.K = 4;
        D.L = 7;
        D.var_e = 3;
        D.var_a1 = 1;
        D.var_a2 = 1;
        D.var_b1 = 0.8;
        D.var_b2 = 0.8;
        D.corr_b = 0.5;
        D.corr_a = 0.3;
        T=[];
        
        border=[0 0.9:0.3:6];
        s_e=[0.001 1 2];
        s_b=[0.001 1 2];
        D.s_a=1;
        
        D.N=D.L*D.K*2;
        [C.dist]=mva_component_examples('3d_sphere',3.5);
        D.P=size(C.dist,1);
        
        % Testing decompositon (again)
        for e=s_e
            for b=s_b
                D.s_e=e;
                D.s_b=b;
                [D,C,Z]=mva_component_examples('cov2_sim',D,C);
                for n=1:100
                    [y,Z,c,trueU]=mva_component_examples('data2_sim',D,C,Z);

                    % mv_pattern
                    Kb=D.K*2;
                    a=0.9;
                    Ac={};
                    Ac{1}=blockdiag(1,0,zeros(Kb,Kb));                                        % Var_a1
                    Ac{2}=blockdiag(0,1,zeros(Kb,Kb));
                    CovCom=chol([1 a;a 1])';
                    Ac{3}=blockdiag(CovCom,zeros(Kb,Kb));                            % Cov_a
                    Ac{4}=blockdiag(zeros(2,2),eye(D.K),zeros(D.K,D.K));                         % var_b1
                    Ac{5}=blockdiag(zeros(2,2),zeros(D.K,D.K),eye(D.K));                         % var_b2
                    Ac{6}=blockdiag(zeros(2,2),eye(Kb))+a*blockdiag(zeros(2,2),diag(ones(D.K,1),D.K)+diag(ones(D.K,1),-D.K));  % Cov_b
                    tic;
                    [G,h,u,l,n]=mvpattern_covcomp(y,Z,'num_iter',1000,'Ac',Ac,'TolL',1e-5,'accel_method','Aitken');
                    D.D1_time=toc;
                    D.D1_F=l(end);
                    D.D1_var_e=h(end,end);
                    D.D1_var_a1=G(1,1);
                    D.D1_var_a2=G(2,2);
                    D.D1_cov_a=G(1,2);
                    D.D1_corr_a=D.D1_cov_a/sqrt(D.D1_var_a1*D.D1_var_a2);
                    
                    D.D1_var_b1=G(3,3);
                    D.D1_var_b2=G(D.K+3,D.K+3);
                    D.D1_cov_b=G(3,D.K+3);
                    D.D1_corr_b=D.D1_cov_b/sqrt(D.D1_var_b1*D.D1_var_b2);
                    
                    [D.D1_kernel_a,D.D1_ksig_a]=spatial_kernel(C.dist,u(1:2,:),border);
                    [D.D1_kernel_b,D.D1_ksig_b]=spatial_kernel(C.dist,u(3:10,:),border);
                    [D.D1_kernel_e,D.D1_ksig_e]=spatial_kernel(C.dist,y-Z*u,border);
                    
                    [D.T_kernel_a,D.T_ksig_a]=spatial_kernel(C.dist,trueU(1:2,:),border);
                    [D.T_kernel_b,D.T_ksig_b]=spatial_kernel(C.dist,trueU(3:10,:),border);
                    [D.T_kernel_e,D.T_ksig_a,D.kernel,D.kernelN]=spatial_kernel(C.dist,y-Z*trueU,border);
                    
                    % Correlation analysis
                    Un=pinv(Z)*y;
                    [D.T_kernel_a,D.T_ksig_a]=spatial_kernel(C.dist,Un(1:2,:),border);
                    [D.T_kernel_b,D.T_ksig_b]=spatial_kernel(C.dist,Un(3:10,:),border);
                    [D.T_kernel_e,D.T_ksig_e]=spatial_kernel(C.dist,y-Z*Un,border);
                    
                    T=addstruct(T,D);
                end;
            end;
        end;
        varargout={T};   
    case 'plot_Figure8'
        T=varargin{1};
        model='D1';
        CAT.linecolor={[0.8 0.8 0.8],[0.4 0.4 0.4],[0 0 0]};
        CAT.errorcolor={[0.8 0.8 0.8],[0.4 0.4 0.4],[0 0 0]};
        CAT.markercolor={[0.8 0.8 0.8],[0.4 0.4 0.4],[0 0 0]};
        
        subplot(3,3,1);
        lineplot(T.s_b,T.([model '_var_a1']),'split',T.s_e, 'style_thickline','leg','auto','CAT',CAT);
        drawline(T.var_a1(1),'dir','horz');
        set(gca,'YLim',[0 1.3]);
        ylabel('var a');
        
        subplot(3,3,2);
        lineplot(T.s_b,T.([model '_corr_a']),'split',T.s_e, 'style_thickline','CAT',CAT);
        drawline(T.corr_a(1),'dir','horz');
        set(gca,'YLim',[0 1]);
        ylabel('corr a');
        
        subplot(3,3,3);
        lineplot(T.s_b,T.([model '_ksig_a']),'split',T.s_e, 'style_thickline','CAT',CAT);
        hold on;
        plot(T.s_b,T.s_a,'k');
        hold off;
        ylabel('s a');
        set(gca,'YLim',[0 2.5]);
        
        subplot(3,3,4);
        lineplot(T.s_b,T.([model '_var_b1']),'split',T.s_e, 'style_thickline','CAT',CAT);
        drawline(T.var_b1(1),'dir','horz');
        set(gca,'YLim',[0 1.3]);
        ylabel('var b');
        
        subplot(3,3,5);
        lineplot(T.s_b,T.([model '_corr_b']),'split',T.s_e, 'style_thickline','CAT',CAT);
        drawline(T.corr_b(1),'dir','horz');
        set(gca,'YLim',[0 1]);
        ylabel('corr_b');
        
        subplot(3,3,6);
        lineplot(T.s_b,T.([model '_ksig_b']),'split',T.s_e, 'style_thickline','CAT',CAT);
        hold on;
        plot(T.s_b,T.s_b,'k');
        hold off;
        ylabel('var b');
        set(gca,'YLim',[0 2.5]);
        
        subplot(3,3,7);
        lineplot(T.s_b,T.([model '_var_e']),'split',T.s_e, 'style_thickline','CAT',CAT);
        drawline(T.var_e(1),'dir','horz');
        set(gca,'YLim',[0 3.5]);
        ylabel('var e');
        
        subplot(3,3,9);
        lineplot(T.s_b,T.([model '_ksig_e']), 'split',T.s_e,'style_thickline','CAT',CAT);
        hold on;
        plot(T.s_b,T.s_e,'k');
        hold off;
        ylabel('var e');
        set(gca,'YLim',[0 2.5]);
        set(gcf,'PaperPosition',[2 2 14 12]/2.54);
        wysiwyg;
end;

function [corrA,corrB]=corrM2coeff(D,C);
corrA=C(1,2);
a=diag(C,D.K);
corrB=mean(a(3:D.K+2));

function err=expkernel_error(sigma,x,r);
rp=exp(-x.^2/(2*sigma.^2));
err=sum((r-rp).^2);

function [uu,sig,xx,N]=spatial_kernel(dist,U,border);
UU=U'*U/size(U,1);
for i=1:length(border)-1
    j=find(dist>=border(i) & dist<border(i+1));
    uu(i)=mean(UU(j));
    N(i)=length(j);
    xx(i)=mean(dist(j));
end;
uu=uu/uu(1);
sig=fminsearch(@(x) expkernel_error(x,xx,uu),1);

